/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { AdminOrganisationsEinheitResource } from '@admin-client/admin-settings';
import { createEmptyStateResource, createStateResource, ProblemDetail } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, Mock, mock, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { ButtonWithSpinnerComponent } from '@ods/component';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { createInvalidParam, createProblemDetail } from '../../../../../../../tech-shared/test/error';
import { createAdminOrganisationsEinheitResource } from '../../../../../test/organisations-einheit/organisations-einheit';
import { OrganisationsEinheitService } from '../../organisationseinheit.service';
import { OrganisationsEinheitFormComponent } from './organisationseinheit-form.component';
import { OrganisationsEinheitSignaturComponent } from './organisationseinheit-signatur/organisationseinheit-signatur.component';
import { OrganisationsEinheitFormService } from './organisationseinheit.formservice';

describe('OrganisationsEinheitFormComponent', () => {
  let component: OrganisationsEinheitFormComponent;
  let fixture: ComponentFixture<OrganisationsEinheitFormComponent>;
  let formService: OrganisationsEinheitFormService;
  let organisationsEinheitService: Mock<OrganisationsEinheitService>;

  const signaturComponentSelector: string = getDataTestIdOf('organisations-einheit-signatur-component');
  const saveButtonSelector: string = getDataTestIdOf('save-button');
  const invalidMessageSpanSelector: string = getDataTestIdOf('invalid-empty-message-span');

  beforeEach(async () => {
    organisationsEinheitService = mock(OrganisationsEinheitService);

    await TestBed.configureTestingModule({
      declarations: [
        OrganisationsEinheitFormComponent,
        MockComponent(OrganisationsEinheitSignaturComponent),
        MockComponent(ButtonWithSpinnerComponent),
      ],
      imports: [ReactiveFormsModule, FormsModule],
      providers: [{ provide: OrganisationsEinheitService, useValue: organisationsEinheitService }],
    }).compileComponents();

    fixture = TestBed.createComponent(OrganisationsEinheitFormComponent);
    component = fixture.componentInstance;
    formService = component.formService;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('input', () => {
    describe('organisationsEinheitStateResource', () => {
      it('should return resource', () => {
        component.updateOrganisationsEinheitResource = jest.fn();

        const organisationsEinheitResource: AdminOrganisationsEinheitResource = createAdminOrganisationsEinheitResource();
        component.organisationsEinheitStateResource = createStateResource(organisationsEinheitResource);

        expect(component.updateOrganisationsEinheitResource).toHaveBeenCalledWith(organisationsEinheitResource);
      });
    });
  });

  describe('component', () => {
    describe('updateOrganisationsEinheitResource', () => {
      it('should call formService patch', () => {
        formService.patch = jest.fn();

        const organisationsEinheitResource: AdminOrganisationsEinheitResource = createAdminOrganisationsEinheitResource();

        component.updateOrganisationsEinheitResource(organisationsEinheitResource);

        expect(formService.patch).toHaveBeenCalledWith(organisationsEinheitResource.settings);
      });
    });

    describe('submit', () => {
      it('should call formService submit', () => {
        formService.submit = jest.fn().mockReturnValue(of(createEmptyStateResource()));

        component.submit();

        expect(formService.submit).toHaveBeenCalled();
      });
    });
  });

  describe('template', () => {
    describe('organisationsEinheit signatur component', () => {
      it('should show signatur component', () => {
        existsAsHtmlElement(fixture, signaturComponentSelector);
      });
    });

    describe('save button', () => {
      it('should show save button', () => {
        existsAsHtmlElement(fixture, saveButtonSelector);
      });
    });

    describe('invalid message', () => {
      it('should show if form is invalid', () => {
        const problemDetail: ProblemDetail = {
          ...createProblemDetail(),
          invalidParams: [{ ...createInvalidParam(), name: 'settingBody.signatur' }],
        };
        formService.setErrorByProblemDetail(problemDetail);

        fixture.detectChanges();

        existsAsHtmlElement(fixture, invalidMessageSpanSelector);
      });

      it('should not show if form valid', () => {
        notExistsAsHtmlElement(fixture, invalidMessageSpanSelector);
      });
    });
  });
});
