/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { fakeAsync, tick } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { cold } from 'jest-marbles';
import { StateResource, createEmptyStateResource } from 'libs/tech-shared/src/lib/resource/resource.util';
import { Dummy, createDummy } from 'libs/tech-shared/test/dummy';
import { singleCold } from 'libs/tech-shared/test/marbles';
import { Observable, of } from 'rxjs';
import * as resourceUtil from '../../../../../tech-shared/src/lib/resource/resource.util';
import { KeycloakResourceService } from './keycloak.resource.service';

describe('KeycloakResourceService', () => {
  let service: KeycloakResourceService<unknown>;

  const id: string = faker.word.sample();
  const emptyStateResource: StateResource<unknown[]> = createEmptyStateResource<unknown[]>();
  const dummyObject: Dummy = createDummy();
  const dummyAction: Observable<Dummy> = of(dummyObject);

  beforeEach(() => {
    service = new TestResourceService();
  });

  describe('get', () => {
    beforeEach(() => {
      service.handleChanges = jest.fn();
    });

    it('should return stateResource as observable', (done) => {
      service.get().subscribe((stateResource) => {
        expect(stateResource).toBe(service.stateResource.value);
        done();
      });
    });

    it('should call handleChanges ', fakeAsync(() => {
      service.get().subscribe();

      expect(service.handleChanges).toHaveBeenCalled();
    }));
  });

  describe('handleChanges', () => {
    it('should call doIfLoadingRequired', () => {
      const doIfLoadingRequired: jest.SpyInstance<boolean> = jest.spyOn(resourceUtil, 'doIfLoadingRequired');

      service.handleChanges(emptyStateResource);

      expect(doIfLoadingRequired).toHaveBeenCalled();
    });
  });

  describe('loadResource', () => {
    it('should call set loading', () => {
      service.setLoading = jest.fn();

      service.loadResource();

      expect(service.setLoading).toHaveBeenCalled();
    });

    it('should update Resource', fakeAsync(() => {
      const dummyItems = [createDummy(), createDummy()];
      service.getItemsFromKeycloak = jest.fn().mockReturnValue(of(dummyItems));

      service.loadResource();
      tick();

      expect(service.stateResource.value.resource).toEqual(dummyItems);
    }));
  });

  describe('create', () => {
    const saveObject: Dummy = createDummy();

    it('should call handleLoading', () => {
      service.handleLoading = jest.fn();

      service.create(saveObject);

      expect(service.handleLoading).toHaveBeenCalled();
    });

    it('should call createInKeycloak', () => {
      service.createInKeycloak = jest.fn().mockReturnValue(of({}));

      service.create(saveObject);

      expect(service.createInKeycloak).toHaveBeenCalled();
    });
  });

  describe('save', () => {
    it('should call handleLoading', () => {
      service.handleLoading = jest.fn();

      service.save(dummyObject);

      expect(service.handleLoading).toHaveBeenCalled();
    });

    it('should call createInKeycloak', () => {
      service.saveInKeycloak = jest.fn().mockReturnValue(of({}));

      service.save(dummyObject);

      expect(service.saveInKeycloak).toHaveBeenCalled();
    });
  });

  describe('delete', () => {
    it('should call handleLoading', () => {
      service.handleLoading = jest.fn();

      service.delete(id);

      expect(service.handleLoading).toHaveBeenCalled();
    });

    it('should call createInKeycloak', () => {
      service.saveInKeycloak = jest.fn().mockReturnValue(of({}));

      service.save(id);

      expect(service.saveInKeycloak).toHaveBeenCalled();
    });
  });

  describe('handleLoading', () => {
    it('should set loading', () => {
      service.handleLoading(dummyAction);

      expect(service.stateResource.value.loading).toBe(true);
    });

    it('should call refreshAfterFirstEmit', () => {
      service.refreshAfterFirstEmit = jest.fn().mockReturnValue(dummyAction);

      service.handleLoading(dummyAction);

      expect(service.refreshAfterFirstEmit).toHaveBeenCalled();
    });

    it('should call progress', () => {
      service.progress = jest.fn().mockReturnValue(dummyAction);

      service.handleLoading(dummyAction);

      expect(service.progress).toHaveBeenCalled();
    });
  });

  describe('refreshAfterFirstEmit', () => {
    it('should call refresh after first emit', fakeAsync(() => {
      service.refresh = jest.fn();

      service.refreshAfterFirstEmit(dummyAction).subscribe();
      tick();

      expect(service.refresh).toHaveBeenCalled();
    }));
  });

  describe('progress', () => {
    it('should emit true at the start and false after first parameter emit', () => {
      const result: Observable<boolean> = service.progress(cold('--x', { x: dummyObject }));

      expect(result).toBeObservable(cold('a-b', { a: true, b: false }));
    });
  });

  describe('setLoading', () => {
    it('should set loading in state to true without parameter', () => {
      service.setLoading();

      expect(service.stateResource.value.loading).toEqual(true);
    });

    it('should set loading in state to false for parameter false', () => {
      service.setLoading(false);

      expect(service.stateResource.value.loading).toEqual(false);
    });
  });

  describe('refresh', () => {
    it('should set reload in state to true', () => {
      service.refresh();

      expect(service.stateResource.value.reload).toBe(true);
    });

    it('should clear resource in state', () => {
      service.refresh();

      expect(service.stateResource.value.resource).toBe(null);
    });
  });

  describe('select resource', () => {
    it('should return state resource', () => {
      const stateResource = createEmptyStateResource<unknown[]>();
      service.stateResource.next(stateResource);

      const resource$: Observable<StateResource<unknown[]>> = service.selectResource();

      expect(resource$).toBeObservable(singleCold(stateResource));
    });
  });
});

class TestResourceService extends KeycloakResourceService<unknown> {
  getItemsFromKeycloak(): Observable<unknown[]> {
    return of(null);
  }

  saveInKeycloak(): Observable<void> {
    return of(null);
  }

  createInKeycloak(): Observable<void> {
    return of(null);
  }

  deleteInKeycloak(): Observable<void> {
    return of(null);
  }
}
