/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { Mock, mock } from '@alfa-client/test-utils';
import { TestBed } from '@angular/core/testing';
import { MatDialogModule } from '@angular/material/dialog';
import { HttpUnauthorizedInterceptor } from './http-unauthorized.interceptor';
import { HttpErrorResponse, HttpHandler, HttpRequest } from '@angular/common/http';
import { Subject, isEmpty } from 'rxjs';
import { AuthenticationService } from './authentication.service';

describe('HttpUnauthorizedInterceptor', () => {
  let interceptor: HttpUnauthorizedInterceptor;

  const authenticationService: Mock<AuthenticationService> = mock(AuthenticationService);

  beforeEach(() =>
    TestBed.configureTestingModule({
      imports: [MatDialogModule],
      providers: [
        HttpUnauthorizedInterceptor,
        {
          provide: AuthenticationService,
          useValue: authenticationService,
        },
      ],
    }),
  );

  beforeEach(() => {
    interceptor = TestBed.inject(HttpUnauthorizedInterceptor);
  });

  it('should be created', () => {
    expect(interceptor).toBeTruthy();
  });

  describe('intercept', () => {
    const error: HttpErrorResponse = new HttpErrorResponse({});

    const handleSubject: Subject<any> = new Subject();
    const httpHandler: HttpHandler = { handle: () => handleSubject };
    const request: HttpRequest<unknown> = new HttpRequest('GET', '/test');

    it('should call handleError with error', () => {
      interceptor.handleError = jest.fn();

      interceptor.intercept(request, httpHandler).subscribe();
      handleSubject.error(error);

      expect(interceptor.handleError).toHaveBeenCalledWith(error);
    });
  });

  describe('handleError', () => {
    describe('on unauthorized status', () => {
      const unauthorizedError: any = new HttpErrorResponse({ status: 401 });

      it('should call logout on authService ', () => {
        interceptor.handleError(unauthorizedError);

        expect(authenticationService.logout).toHaveBeenCalled();
      });

      it('should return EMPTY', (done) => {
        interceptor
          .handleError(unauthorizedError)
          .pipe(isEmpty())
          .subscribe((res) => {
            expect(res).toBeTruthy;
            done();
          });
      });
    });

    it('should rethrow error if not unauthorized status', () => {
      const anyError: any = new HttpErrorResponse({ status: 500 });

      expect(() => interceptor.handleError(anyError)).toThrowError();
    });
  });
});
