/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidService } from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import { StateResource, createErrorStateResource } from '@alfa-client/tech-shared';
import {
  DialogRefMock,
  Mock,
  createDialogRefMock,
  existsAsHtmlElement,
  getElementComponentFromFixtureByCss,
  mock,
  triggerEvent,
} from '@alfa-client/test-utils';
import { OzgcloudStrokedButtonWithSpinnerComponent } from '@alfa-client/ui';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { createBescheidResource } from '../../../../../../bescheid-shared/src/test/bescheid';
import {
  createCommandStateResource,
  createSuccessfullyDoneCommandStateResource,
} from '../../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import { createApiError } from '../../../../../../tech-shared/test/error';
import { singleColdCompleted } from '../../../../../../tech-shared/test/marbles';
import { BescheidFormService } from '../../bescheid.formservice';
import {
  BescheidWizardCancelDialogContainerComponent,
  CancelWizardDialogData,
} from './bescheid-wizard-cancel-dialog-container.component';

describe('BescheidWizardCancelDialogContainerComponent', () => {
  let component: BescheidWizardCancelDialogContainerComponent;
  let fixture: ComponentFixture<BescheidWizardCancelDialogContainerComponent>;

  const speichernButton: string = getDataTestIdOf('bescheiderstellung-abbrechen-entwurf-speichern');
  const verwerfenButton: string = getDataTestIdOf('bescheiderstellung-abbrechen-entwurf-verwerfen');

  const dialogData: CancelWizardDialogData = {
    bescheidResource: createBescheidResource(),
  };

  let formService: Mock<BescheidFormService>;
  let bescheidService: Mock<BescheidService>;
  let dialogRef: DialogRefMock;

  beforeEach(() => {
    formService = { ...mock(BescheidFormService), submit: jest.fn().mockReturnValue(EMPTY) };
    bescheidService = { ...mock(BescheidService), bescheidVerwerfen: jest.fn().mockReturnValue(EMPTY) };
    dialogRef = createDialogRefMock();
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [BescheidWizardCancelDialogContainerComponent, MockComponent(OzgcloudStrokedButtonWithSpinnerComponent)],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: BescheidFormService,
          useValue: formService,
        },
        {
          provide: DIALOG_DATA,
          useValue: dialogData,
        },
        {
          provide: DialogRef,
          useValue: dialogRef,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardCancelDialogContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('save', () => {
      beforeEach(() => {
        component.closeDialogWithWizard = jest.fn();
      });

      it('should submit form', () => {
        component.save();

        expect(formService.submit).toHaveBeenCalled();
      });

      it('should emit save command state resource', () => {
        const commandStateResource: StateResource<CommandResource> = createCommandStateResource();
        formService.submit = jest.fn().mockReturnValue(of(commandStateResource));

        component.save();

        expect(component.saveStateResource$).toBeObservable(singleColdCompleted(commandStateResource));
      });

      it('should handle successfully saved bescheid', () => {
        const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
        formService.submit = jest.fn().mockReturnValue(of(commandStateResource));

        component.save();
        component.saveStateResource$.subscribe();

        expect(component.closeDialogWithWizard).toHaveBeenCalled();
      });

      it('should NOT handle successfully saved bescheid', () => {
        const commandStateResource: StateResource<CommandResource> = createErrorStateResource(createApiError());
        formService.submit = jest.fn().mockReturnValue(of(commandStateResource));

        component.save();
        component.saveStateResource$.subscribe();

        expect(component.closeDialogWithWizard).not.toHaveBeenCalled();
      });
    });

    describe('cancel', () => {
      beforeEach(() => {
        component.closeDialogWithWizard = jest.fn();
      });

      it('should delete bescheid', () => {
        component.cancel();

        expect(bescheidService.bescheidVerwerfen).toHaveBeenCalled();
      });

      it('should NOT delete bescheid', () => {
        component.bescheidResource = null;

        component.cancel();

        expect(bescheidService.bescheidVerwerfen).not.toHaveBeenCalled();
      });

      it('should emit delete command state resource', () => {
        const commandStateResource: StateResource<CommandResource> = createCommandStateResource();
        bescheidService.bescheidVerwerfen = jest.fn().mockReturnValue(of(commandStateResource));

        component.cancel();

        expect(component.deleteStateResource$).toBeObservable(singleColdCompleted(commandStateResource));
      });

      it('should close dialog on success', () => {
        const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
        bescheidService.bescheidVerwerfen = jest.fn().mockReturnValue(of(commandStateResource));

        component.cancel();
        component.deleteStateResource$.subscribe();

        expect(component.closeDialogWithWizard).toHaveBeenCalled();
      });

      it('should NOT close dialog on error', () => {
        const commandStateResource: StateResource<CommandResource> = createErrorStateResource(createApiError());
        bescheidService.bescheidVerwerfen = jest.fn().mockReturnValue(of(commandStateResource));

        component.cancel();
        component.deleteStateResource$.subscribe();

        expect(component.closeDialogWithWizard).not.toHaveBeenCalled();
      });

      it('should close dialog with cancel result on non existing bescheid', () => {
        component.bescheidResource = null;

        component.cancel();

        expect(component.closeDialogWithWizard).toHaveBeenCalled();
      });
    });

    describe('closeDialogWithWizard', () => {
      it('should close', () => {
        component.closeDialogWithWizard();

        expect(dialogRef.close).toHaveBeenCalledWith({ closeWizard: true });
      });
    });
  });

  describe('template', () => {
    describe('ozgcloud-stroked-button-with-spinner Entwurf speichern', () => {
      it('should show', () => {
        existsAsHtmlElement(fixture, speichernButton);
      });

      describe('input', () => {
        it('should set stateResource', () => {
          const commandStateResource: StateResource<CommandResource> = createCommandStateResource();
          component.saveStateResource$ = of(commandStateResource);

          const elementComponent: OzgcloudStrokedButtonWithSpinnerComponent = getElementComponentFromFixtureByCss(
            fixture,
            speichernButton,
          );

          fixture.detectChanges();

          expect(elementComponent.stateResource).toBe(commandStateResource);
        });
      });

      describe('output', () => {
        it('should call save', () => {
          component.save = jest.fn();

          triggerEvent({ fixture, name: 'clickEmitter', elementSelector: speichernButton });

          expect(component.save).toHaveBeenCalled();
        });
      });
    });

    describe('ozgcloud-stroked-button-with-spinner Verwerfen', () => {
      it('should show', () => {
        existsAsHtmlElement(fixture, verwerfenButton);
      });

      describe('input', () => {
        it('should set stateResource', () => {
          const commandStateResource: StateResource<CommandResource> = createCommandStateResource();
          component.deleteStateResource$ = of(commandStateResource);

          const elementComponent: OzgcloudStrokedButtonWithSpinnerComponent = getElementComponentFromFixtureByCss(
            fixture,
            verwerfenButton,
          );

          fixture.detectChanges();

          expect(elementComponent.stateResource).toBe(commandStateResource);
        });
      });

      describe('output', () => {
        it('should call delete', () => {
          component.cancel = jest.fn();

          triggerEvent({ fixture, name: 'clickEmitter', elementSelector: verwerfenButton });

          expect(component.cancel).toHaveBeenCalled();
        });
      });
    });
  });
});
