/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { HttpErrorResponse } from '@angular/common/http';
import { createApiError, createHttpErrorResponse, createIssue } from 'libs/tech-shared/test/error';
import { MessageCode } from '../message-code';
import { ApiError, Issue } from '../tech.model';
import {
  getApiErrorFromHttpErrorResponse,
  isApiError,
  isNotApiError,
  isResourceNotFoundError,
  isServiceUnavailableMessageCode,
} from './error.util';

describe('error.util', () => {
  describe('isApiError', () => {
    it('should return true if parameter is ApiError', () => {
      const apiError: ApiError = createApiError();

      const result: boolean = isApiError(apiError);

      expect(result).toBeTruthy();
    });

    it('should return false if empty ApiError.Issues property', () => {
      const apiError: ApiError = createApiError([]);

      const result: boolean = isApiError(apiError);

      expect(result).toBeFalsy();
    });

    it('should return false if parameter is no ApiError', () => {
      const httpErrorResponse: HttpErrorResponse = createHttpErrorResponse();

      const result: boolean = isApiError(httpErrorResponse);

      expect(result).toBeFalsy();
    });
  });

  describe('isNotApiError', () => {
    it('should return true if parameter is no ApiError', () => {
      const result: boolean = isNotApiError(createHttpErrorResponse());

      expect(result).toBeTruthy();
    });

    it('should return false if parameter is ApiError', () => {
      const issue: Issue = createIssue();
      const apiError: ApiError = createApiError([issue]);

      const result: boolean = isNotApiError(apiError);

      expect(result).toBeFalsy();
    });
  });

  describe('isServiceUnavailableMessageCode', () => {
    it('should return true if SERVICE_UNAVAILABLE error code', () => {
      const issue: Issue = { ...createIssue(), messageCode: MessageCode.SERVICE_UNAVAILABLE };
      const apiError: ApiError = createApiError([issue]);

      const result: boolean = isServiceUnavailableMessageCode(apiError);

      expect(result).toBeTruthy();
    });

    it('should return false if not SERVICE_UNAVAILABLE error code', () => {
      const issue: Issue = createIssue();
      const apiError: ApiError = createApiError([issue]);

      const result: boolean = isServiceUnavailableMessageCode(apiError);

      expect(result).toBeFalsy();
    });

    it('should return false if input parameter is no ApiError', () => {
      const httpErrorResponse: HttpErrorResponse = new HttpErrorResponse({});

      const result: boolean = isServiceUnavailableMessageCode(httpErrorResponse);

      expect(result).toBeFalsy();
    });
  });

  describe('isResourceNotFoundError', () => {
    it('should return true if RESOURCE_NOT_FOUND error code', () => {
      const issue: Issue = { ...createIssue(), messageCode: MessageCode.RESOURCE_NOT_FOUND };
      const apiError: ApiError = createApiError([issue]);

      const result: boolean = isResourceNotFoundError(apiError);

      expect(result).toBeTruthy();
    });

    it('should return false if not RESOURCE_NOT_FOUND error code', () => {
      const issue: Issue = createIssue();
      const apiError: ApiError = createApiError([issue]);

      const result: boolean = isResourceNotFoundError(apiError);

      expect(result).toBeFalsy();
    });

    it('should return false if input parameter is no ApiError', () => {
      const httpErrorResponse: HttpErrorResponse = new HttpErrorResponse({});

      const result: boolean = isResourceNotFoundError(httpErrorResponse);

      expect(result).toBeFalsy();
    });
  });

  describe('getApiErrorFromHttpErrorResponse', () => {
    it('should return ApiError', () => {
      const issue: Issue = createIssue();
      const apiError: ApiError = createApiError([issue]);
      const httpErrorResponse: HttpErrorResponse = createHttpErrorResponse(apiError);

      const result: ApiError = getApiErrorFromHttpErrorResponse(httpErrorResponse);

      expect(result).toBe(apiError);
    });

    it('should return undefined if no ApiError', () => {
      const httpErrorResponse: HttpErrorResponse = new HttpErrorResponse({});

      const result: ApiError = getApiErrorFromHttpErrorResponse(httpErrorResponse);

      expect(result).toBeUndefined();
    });
  });
});
