/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.time.ZonedDateTime;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.alfa.common.binaryfile.FileIdMapper;
import de.ozgcloud.alfa.common.user.UserId;
import de.ozgcloud.alfa.common.user.UserIdMapper;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class BescheidMapperTest {

	@Mock
	private FileIdMapper fileIdMapper;
	@Mock
	private UserIdMapper userIdMapper;
	@InjectMocks
	private final BescheidMapper mapper = Mappers.getMapper(BescheidMapper.class);

	@Nested
	class TestFromGrpc {

		@Test
		void shouldMapId() {
			var bescheid = map();

			assertThat(bescheid.getId()).isEqualTo(GrpcBescheidTestFactory.ID);
		}

		@Test
		void shouldMapVorgangId() {
			var bescheid = map();

			assertThat(bescheid.getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldMapBewilligt() {
			var bescheid = map();

			assertThat(bescheid.getBewilligt()).isEqualTo(GrpcBescheidTestFactory.BEWILLIGT);
		}

		@Test
		void shouldMapBeschiedenAm() {
			var bescheid = map();

			assertThat(bescheid.getBeschiedenAm()).isEqualTo(GrpcBescheidTestFactory.BESCHIEDEN_AM);
		}

		@Test
		void shouldMapBescheidDocument() {
			var bescheid = map();

			assertThat(bescheid.getBescheidDocument()).isEqualTo(GrpcBescheidTestFactory.BESCHEID_DOCUMENT);
		}

		@Test
		void shouldMapAttachments() {
			when(fileIdMapper.toFileId(any())).thenCallRealMethod();

			var bescheid = map();

			assertThat(bescheid.getAttachments()).containsExactlyInAnyOrderElementsOf(BescheidTestFactory.ATTACHMENTS);
		}

		@Test
		void shouldMapNachrichtText() {
			var bescheid = map();

			assertThat(bescheid.getNachrichtText()).isEqualTo(GrpcBescheidTestFactory.NACHRICHT_TEXT);
		}

		@Test
		void shouldMapNachrichtSubject() {
			var bescheid = map();

			assertThat(bescheid.getNachrichtSubject()).isEqualTo(GrpcBescheidTestFactory.NACHRICHT_SUBJECT);
		}

		@Test
		void shouldMapStatus() {
			var bescheid = map();

			assertThat(bescheid.getStatus()).isEqualTo(BescheidTestFactory.STATUS);
		}

		@Nested
		class TestMapSentInfo {
			private final SentInfo expectedSentInfo = SentInfoTestFactory.createBuilder()
					.sentAt(ZonedDateTime.parse(GrpcSentInfoTestFactory.SENT_AT))
					.sentBy(UserId.from(GrpcSentInfoTestFactory.SENT_BY))
					.build();

			@Test
			void shouldMapSentBy() {
				when(userIdMapper.fromString(GrpcSentInfoTestFactory.SENT_BY)).thenCallRealMethod();

				var bescheid = map();

				assertThat(bescheid.getSentInfo().getSentBy()).isEqualTo(expectedSentInfo.getSentBy());
			}

			@Test
			void shouldMapSentAt() {
				var bescheid = map();

				assertThat(bescheid.getSentInfo().getSentAt()).isEqualTo(expectedSentInfo.getSentAt());
			}
		}

		private Bescheid map() {
			return mapper.fromGrpc(GrpcBescheidTestFactory.create(), VorgangHeaderTestFactory.ID);
		}
	}
}
