/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.Spy;

import de.ozgcloud.collaboration.request.GrpcCollaborationRequest;

class CollaborationMapperTest {

	@Spy
	private CollaborationMapper mapper = Mappers.getMapper(CollaborationMapper.class);

	@Nested
	class TestFromCollaborationRequest {

		@Nested
		class OnOrganisationsEinheitRequest {

			public final GrpcCollaborationRequest request = GrpcFindRequestsResponseTestFactory.COLLABORATION_REQUEST;
			public final OrganisationsEinheitCollaboration collaboration = OrganisationsEinheitCollaborationTestFactory.create();

			@BeforeEach
			void init() {
				doReturn(false).when(mapper).isFachstelle(request);
				doReturn(collaboration).when(mapper).fromOrganisationsEinheitRequest(request);
			}

			@Test
			void shouldCheckForFachstelleRequest() {
				mapper.fromCollaborationRequest(request);

				verify(mapper).isFachstelle(request);
			}

			@Test
			void shouldMapToOrganisationsEinheitCollaboration() {
				mapper.fromCollaborationRequest(request);

				verify(mapper).fromOrganisationsEinheitRequest(request);
			}

			@Test
			void shouldReturnOrganisationsEinheitCollaboration() {
				var mapped = mapper.fromCollaborationRequest(request);

				assertThat(mapped).isEqualTo(collaboration);
			}
		}

		@Nested
		class OnFachstelleRequest {

			public final GrpcCollaborationRequest request = GrpcFindRequestsResponseTestFactory.COLLABORATION_REQUEST;
			public final FachstelleCollaboration collaboration = FachstelleCollaborationTestFactory.create();

			@BeforeEach
			void init() {
				doReturn(true).when(mapper).isFachstelle(request);
				doReturn(collaboration).when(mapper).fromFachstelleRequest(request);
			}

			@Test
			void shouldCheckForFachstelleRequest() {
				mapper.fromCollaborationRequest(request);

				verify(mapper).isFachstelle(request);
			}

			@Test
			void shouldMapToFachstelleCollaboration() {
				mapper.fromCollaborationRequest(request);

				verify(mapper).fromFachstelleRequest(request);
			}

			@Test
			void shouldReturnFachstelleCollaboration() {
				var mapped = mapper.fromCollaborationRequest(request);

				assertThat(mapped).isEqualTo(collaboration);
			}
		}
	}

	@Nested
	class TestIsFachstelle {

		@Test
		void shouldReturnTrue() {
			var fachstelle = mapper.isFachstelle(GrpcCollaborationRequestForFachstelleTestFactory.create());

			assertThat(fachstelle).isTrue();
		}

		@Test
		void shouldReturnFalse() {
			var fachstelle = mapper.isFachstelle(GrpcCollaborationRequestForOrganisationsEinheitTestFactory.create());

			assertThat(fachstelle).isFalse();
		}
	}

	@Nested
	class TestFromOrganisationsEinheitRequest {

		@Test
		void shouldMap() {
			var collaboration = mapper.fromOrganisationsEinheitRequest(GrpcCollaborationRequestForOrganisationsEinheitTestFactory.create());

			assertThat(collaboration).usingRecursiveComparison().isEqualTo(OrganisationsEinheitCollaborationTestFactory.create());
		}
	}

	@Nested
	class TestFromFachstelleRequest {

		@Test
		void shouldMap() {
			var collaboration = mapper.fromFachstelleRequest(GrpcCollaborationRequestForFachstelleTestFactory.create());

			assertThat(collaboration).usingRecursiveComparison().isEqualTo(FachstelleCollaborationTestFactory.create());
		}
	}
}
