/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.collaboration.request.CollaborationRequestServiceGrpc.CollaborationRequestServiceBlockingStub;
import de.ozgcloud.collaboration.request.GrpcFindRequestsRequest;
import de.ozgcloud.collaboration.request.GrpcGetRequestRequest;

class CollaborationRemoteServiceTest {

	@Spy
	@InjectMocks
	private CollaborationRemoteService service;
	@Mock
	private CollaborationRequestServiceBlockingStub stub;
	@Mock
	private CollaborationMapper mapper;

	@Nested
	class TestGetById {

		public static final String COLLABORATION_ID = GrpcGetRequestRequestTestFactory.COLLABORATION_ID;

		private final GrpcGetRequestRequest request = GrpcGetRequestRequestTestFactory.create();
		private final Collaboration collaboration = OrganisationsEinheitCollaborationTestFactory.create();

		@BeforeEach
		void init() {
			when(stub.getRequest(request)).thenReturn(GrpcGetRequestResponseTestFactory.create());
			when(mapper.fromCollaborationRequest(GrpcGetRequestResponseTestFactory.COLLABORATION_REQUEST)).thenReturn(collaboration);
		}

		@Test
		void shouldBuildGetRequest() {
			service.getById(COLLABORATION_ID);

			verify(service).buildGetRequest(COLLABORATION_ID);
		}

		@Test
		void shouldGetRequest() {
			service.getById(COLLABORATION_ID);

			verify(stub).getRequest(request);
		}

		@Test
		void shouldMapFromCollaborationRequest() {
			service.getById(COLLABORATION_ID);

			verify(mapper).fromCollaborationRequest(GrpcGetRequestResponseTestFactory.COLLABORATION_REQUEST);
		}

		@Test
		void shouldReturnCollaboration() {
			var collaborationById = service.getById(COLLABORATION_ID);

			assertThat(collaborationById).isEqualTo(collaboration);
		}
	}

	@Nested
	class TestGetCollaborations {

		public static final String VORGANG_ID = VorgangHeaderTestFactory.ID;

		private final GrpcFindRequestsRequest searchRequest = GrpcFindRequestsRequestTestFactory.create();
		private final Collaboration collaboration = OrganisationsEinheitCollaborationTestFactory.create();

		@BeforeEach
		void init() {
			doReturn(searchRequest).when(service).buildSearchRequest(VORGANG_ID);
			when(stub.findRequests(searchRequest)).thenReturn(GrpcFindRequestsResponseTestFactory.create());
			when(mapper.fromCollaborationRequest(GrpcFindRequestsResponseTestFactory.COLLABORATION_REQUEST)).thenReturn(collaboration);
		}

		@Test
		void shouldBuildSearchRequest() {
			service.getCollaborations(VORGANG_ID).toList();

			verify(service).buildSearchRequest(VORGANG_ID);
		}

		@Test
		void shouldFindRequests() {
			service.getCollaborations(VORGANG_ID).toList();

			verify(stub).findRequests(searchRequest);
		}

		@Test
		void shouldMapFromCollaborationRequest() {
			service.getCollaborations(VORGANG_ID).toList();

			verify(mapper).fromCollaborationRequest(GrpcFindRequestsResponseTestFactory.COLLABORATION_REQUEST);
		}


		@Test
		void shouldReturnCollaborations() {
			var collaborations = service.getCollaborations(VORGANG_ID).toList();

			assertThat(collaborations).containsExactly(collaboration);
		}
	}

	@Nested
	class TestBuildGetRequest {

		@Test
		void shouldSetCollaborationId() {
			var request = service.buildGetRequest(GrpcGetRequestRequestTestFactory.COLLABORATION_ID);

			assertThat(request.getId()).isEqualTo(GrpcGetRequestRequestTestFactory.COLLABORATION_ID);
		}
	}

	@Nested
	class TestBuildSearchRequest {

		@Test
		void shouldSetVorgangId() {
			var request = service.buildSearchRequest(GrpcFindRequestsRequestTestFactory.VORGANG_ID);

			assertThat(request).usingRecursiveComparison().isEqualTo(GrpcFindRequestsRequestTestFactory.create());
		}
	}
}
