/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.alfa.common.TimeMapper;
import de.ozgcloud.alfa.common.callcontext.CallContextMapper;
import de.ozgcloud.alfa.common.user.UserIdMapper;
import de.ozgcloud.alfa.vorgang.RedirectRequestTestFactory;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.grpc.command.GrpcCommand;
import de.ozgcloud.vorgang.grpc.command.GrpcRedirectRequest;

class CommandMapperTest {

	@InjectMocks
	private CommandMapper mapper = Mappers.getMapper(CommandMapper.class);
	@Mock
	private UserIdMapper userIdMapper;
	@Mock
	private GrpcObjectMapper grpcObjectMapper;
	@Mock
	private CommandBodyMapper bodyMapper;
	@Mock
	private TimeMapper timeMapper;
	@Mock
	private CallContextMapper callContextMapper;

	@Test
	void shouldMapNonEnumOrder() {
		var command = mapper.toCommand(GrpcCommandTestFactory.createBuilder()
				.setOrderString(CommandOrder.SEND_POSTFACH_NACHRICHT.name())
				.build());

		assertThat(command.getCommandOrder()).isEqualTo(CommandOrder.SEND_POSTFACH_NACHRICHT);
	}

	@DisplayName("Map from grpc to command")
	@Nested
	class TestToCommand {

		@DisplayName("redirect request")
		@Nested
		class TestRedirectRequest {

			@Test
			void shouldMapPasswordIfExists() {
				var command = mapper.toCommand(GrpcCommandTestFactory.createBuilder()
						.setRedirectRequest(GrpcRedirectRequest.newBuilder()
								.setPassword(RedirectRequestTestFactory.PASSWORD_AS_STR)
								.setEmail(RedirectRequestTestFactory.EMAIL)
								.build())
						.build());

				assertThat(command.getRedirectRequest().getPassword()).isEqualTo(RedirectRequestTestFactory.PASSWORD);
				assertThat(command.getRedirectRequest().getEmail()).isEqualTo(RedirectRequestTestFactory.EMAIL);
			}

			@Test
			void shouldNotMapPasswordIfEmpty() {
				var command = mapper.toCommand(GrpcCommandTestFactory.createBuilder()
						.setRedirectRequest(GrpcRedirectRequest.newBuilder()
								.setEmail(RedirectRequestTestFactory.EMAIL)
								.build())
						.build());

				assertThat(command.getRedirectRequest().getPassword()).isNull();
				assertThat(command.getRedirectRequest().getEmail()).isEqualTo(RedirectRequestTestFactory.EMAIL);
			}
		}
	}

	@DisplayName("Map Body")
	@Nested
	class TestMapBody {

		private GrpcCommand command = GrpcCommandTestFactory.create();

		@DisplayName("on existing body object at command")
		@Nested
		class TestOnExistingBodyObject {

			@Test
			void shouldCallGrpcObjectMapper() {
				mapper.mapBody(command);

				verify(grpcObjectMapper).mapFromGrpc(command.getBodyObj());
			}

			@Test
			void shouldMapFromCommandBodyObject() {
				var bodyObjectMap = Map.<String, Object>of("test-key", "test-value");
				when(grpcObjectMapper.mapFromGrpc(any())).thenReturn(bodyObjectMap);

				var result = mapper.mapBody(command);

				assertThat(result).isEqualTo(bodyObjectMap);
			}
		}

		@DisplayName("on empty body object at command")
		@Nested
		class TestOnEmptyBodyObject {

			private Map<String, String> mappedBody = Map.of("test-key", "test-value");

			@BeforeEach
			void mockMapper() {
				when(grpcObjectMapper.mapFromGrpc(any())).thenReturn(Collections.emptyMap());
				when(bodyMapper.map(any())).thenReturn(mappedBody);
			}

			@Test
			void shouldCallBodyMapper() {
				mapper.mapBody(command);

				verify(bodyMapper).map(command.getBody());
			}

			@Test
			void shouldReturnMappedResult() {
				var result = mapper.mapBody(command);

				assertThat(result).isEqualTo(mappedBody);
			}
		}
	}
}