/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang.command;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.junit.jupiter.params.provider.ValueSource;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;

import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.Vorgang;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class VorgangCommandProcessorTest {

	private static final String EXPECTED_COMMAND_LINK_TEMPL = "/api/vorgangs/%s/relations/%s/%d/commands";
	private static final String EXPECTED_COMMAND_LINK = String.format(EXPECTED_COMMAND_LINK_TEMPL, CommandTestFactory.VORGANG_ID,
			VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.VERSION);

	@Spy
	@InjectMocks
	private VorgangCommandProcessor processor;
	@Mock
	private CurrentUserService userService;

	@Nested
	@DisplayName("add 'annehmen' link")
	class TestAnnehmenLink {

		private final LinkRelation linkRel = VorgangCommandProcessor.REL_VORGANG_ANNEHMEN;

		@Test
		void shouldBePresent() {
			var processed = processor.process(buildVorgangInStatus(VorgangStatus.NEU));

			assertThat(processed.getLink(linkRel)).isPresent().get()
					.extracting(Link::getHref).isEqualTo(EXPECTED_COMMAND_LINK);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "NEU" })
		void shouldNOTbePresentbyStatus(VorgangStatus status) {
			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLink(linkRel)).isEmpty();
		}
	}

	@Nested
	@DisplayName("add 'verwerfen' link")
	class TestVerwerfenLink {

		private final LinkRelation linkRel = VorgangCommandProcessor.REL_VORGANG_VERWERFEN;

		@ParameterizedTest
		@ValueSource(strings = { "EINHEITLICHER_ANSPRECHPARTNER", "VERWALTUNG_USER" })
		void shouldBePresentAtStatusAndRole(String userRole) {
			doReturn(userRole).when(processor).getUserRole();

			var processed = processor.process(buildVorgangInStatus(VorgangStatus.NEU));

			assertThat(processed.getLink(linkRel)).isPresent().get()
					.extracting(Link::getHref).isEqualTo(EXPECTED_COMMAND_LINK);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "NEU" })
		void shouldNOTbePresentAtStatusAndRoleEa(VorgangStatus status) {
			doReturn(UserRole.EINHEITLICHER_ANSPRECHPARTNER).when(processor).getUserRole();

			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLink(linkRel)).isEmpty();
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "NEU" })
		void shouldNOTbePresentAtStatusAndRoleVerwaltungUser(VorgangStatus status) {
			doReturn(UserRole.VERWALTUNG_USER).when(processor).getUserRole();

			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLink(linkRel)).isEmpty();
		}
	}

	@Nested
	@DisplayName("add 'zurueckholen' link")
	class TestZurueckholenLink {

		private final LinkRelation linkRel = VorgangCommandProcessor.REL_VORGANG_ZURUECKHOLEN;

		@ParameterizedTest
		@ValueSource(strings = { "EINHEITLICHER_ANSPRECHPARTNER", "VERWALTUNG_USER" })
		void shouldBePresentAtStatusAndRole(String userRole) {
			var processed = processor.process(buildVorgangInStatus(VorgangStatus.VERWORFEN));

			assertThat(processed.getLink(linkRel)).isPresent().get()
					.extracting(Link::getHref).isEqualTo(EXPECTED_COMMAND_LINK);
		}

		@ParameterizedTest
		@EnumSource(names = { "NEU", "ANGENOMMEN" })
		void shouldNOTbePresentAtStatusAndRoleVerwaltungUser(VorgangStatus status) {
			doReturn(UserRole.VERWALTUNG_USER).when(processor).getUserRole();

			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLink(linkRel)).isEmpty();
		}

		@ParameterizedTest
		@EnumSource(names = { "NEU", "ANGENOMMEN" })
		void shouldNOTbePresentAtStatusAndRoleEa(VorgangStatus status) {
			doReturn(UserRole.EINHEITLICHER_ANSPRECHPARTNER).when(processor).getUserRole();

			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLink(linkRel)).isEmpty();
		}
	}

	@Nested
	@DisplayName("add 'bearbeiten' link")
	class TestBearbeitenLink {

		private final LinkRelation linkRel = VorgangCommandProcessor.REL_VORGANG_BEARBEITEN;

		@Test
		void shouldBePresent() {
			var processed = processor.process(buildVorgangInStatus(VorgangStatus.ANGENOMMEN));

			assertThat(processed.getLink(linkRel)).isPresent().get()
					.extracting(Link::getHref)
					.isEqualTo(EXPECTED_COMMAND_LINK);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "ANGENOMMEN", "ABGESCHLOSSEN" })
		void shouldNOTbePresentbyStatus(VorgangStatus status) {
			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLink(linkRel)).isEmpty();
		}
	}

	@Nested
	@DisplayName("add 'bescheiden' link")
	class TestBescheidenLink {

		private final LinkRelation linkRel = VorgangCommandProcessor.REL_VORGANG_BESCHEIDEN;

		@Test
		void shouldBePresent() {
			var processed = processor.process(buildVorgangInStatus(VorgangStatus.IN_BEARBEITUNG));

			assertThat(processed.getLink(linkRel)).isPresent().get()
					.extracting(Link::getHref)
					.isEqualTo(EXPECTED_COMMAND_LINK);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "IN_BEARBEITUNG" })
		void shouldNOTbePresentbyStatus(VorgangStatus status) {
			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLink(linkRel)).isEmpty();
		}
	}

	@Nested
	@DisplayName("add 'zurueckstellen' link")
	class TestZurueckstellenLink {

		private final LinkRelation linkRel = VorgangCommandProcessor.REL_VORGANG_ZURUECKSTELLEN;

		@Test
		void shouldBePresent() {
			var processed = processor.process(buildVorgangInStatus(VorgangStatus.IN_BEARBEITUNG));

			assertThat(processed.getLink(linkRel)).isPresent().get()
					.extracting(Link::getHref).isEqualTo(EXPECTED_COMMAND_LINK);
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "IN_BEARBEITUNG" })
		void shouldNOTbePresentbyStatus(VorgangStatus status) {
			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLink(linkRel)).isEmpty();
		}
	}

	@Nested
	@DisplayName("add 'abschliessen' link")
	class TestAbschliessenLink {

		private final LinkRelation linkRel = VorgangCommandProcessor.REL_VORGANG_ABSCHLIESSEN;

		@Nested
		class TestOnVerwaltungUser {

			@BeforeEach
			void mockUserRole() {
				doReturn(UserRole.VERWALTUNG_USER).when(processor).getUserRole();
			}

			@ParameterizedTest
			@EnumSource(mode = Mode.INCLUDE, names = { "IN_BEARBEITUNG", "BESCHIEDEN" })
			void shouldBePresent(VorgangStatus status) {
				var processed = processor.process(buildVorgangInStatus(status));

				assertThat(processed.getLink(linkRel)).isPresent().get()
						.extracting(Link::getHref)
						.isEqualTo(EXPECTED_COMMAND_LINK);
			}

			@ParameterizedTest
			@EnumSource(mode = Mode.EXCLUDE, names = { "IN_BEARBEITUNG", "BESCHIEDEN" })
			void shouldNOTbePresentbyStatus(VorgangStatus status) {
				var processed = processor.process(buildVorgangInStatus(status));

				assertThat(processed.getLink(linkRel)).isEmpty();
			}
		}

		@Nested
		class TestOnEinheitlicherAnsprechpartner {

			@BeforeEach
			void mockUserRole() {
				doReturn(UserRole.EINHEITLICHER_ANSPRECHPARTNER).when(processor).getUserRole();
			}

			@Test
			void shouldBePresent() {
				var processed = processor.process(buildVorgangInStatus(VorgangStatus.IN_BEARBEITUNG));

				assertThat(processed.getLink(linkRel)).isPresent().get()
						.extracting(Link::getHref)
						.isEqualTo(EXPECTED_COMMAND_LINK);
			}

			@ParameterizedTest
			@EnumSource(mode = Mode.EXCLUDE, names = { "IN_BEARBEITUNG" })
			void shouldNOTbePresentbyStatus(VorgangStatus status) {
				var processed = processor.process(buildVorgangInStatus(status));

				assertThat(processed.getLink(linkRel)).isEmpty();
			}
		}
	}

	@Nested
	@DisplayName("add 'wiedereroeffnen' link")
	class TestWiedereroeffnenLink {

		private final LinkRelation linkRel = VorgangCommandProcessor.REL_VORGANG_WIEDEREROEFFNEN;

		@DisplayName("on Vorgang with Status Abgeschlossen")
		@ParameterizedTest
		@ValueSource(strings = { "EINHEITLICHER_ANSPRECHPARTNER", "VERWALTUNG_USER" })
		void shouldBePresentOnStatusAbgeschlossen(String userRole) {
			doReturn(userRole).when(processor).getUserRole();

			var processed = processor.process(buildVorgangInStatus(VorgangStatus.ABGESCHLOSSEN));

			assertThat(processed.getLink(linkRel)).isPresent().get()
					.extracting(Link::getHref).isEqualTo(EXPECTED_COMMAND_LINK);
		}

		@Test
		@DisplayName("on Vorgang with Status Beschieden and Role VerwaltungUser")
		void shouldBePresentOnStatusBeschieden() {
			var processed = processor.process(buildVorgangInStatus(VorgangStatus.BESCHIEDEN));

			assertThat(processed.getLink(linkRel)).isPresent().get()
					.extracting(Link::getHref).isEqualTo(EXPECTED_COMMAND_LINK);
		}

		@Test
		@DisplayName("on Vorgang with Status Beschieden and Role EA")
		void shouldBeNotPresentOnStatusBeschiedenAndRoleEA() {
			doReturn(UserRole.EINHEITLICHER_ANSPRECHPARTNER).when(processor).getUserRole();

			var processed = processor.process(buildVorgangInStatus(VorgangStatus.BESCHIEDEN));

			assertThat(processed.getLink(linkRel)).isNotPresent();
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "ABGESCHLOSSEN", "BESCHIEDEN" })
		void shouldNOTbePresentbyStatusAndRoleVerwaltungUser(VorgangStatus status) {
			doReturn(UserRole.VERWALTUNG_USER).when(processor).getUserRole();

			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLink(linkRel)).isEmpty();
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "ABGESCHLOSSEN", "BESCHIEDEN" })
		void shouldNOTbePresentbyStatusAndRoleEA(VorgangStatus status) {
			doReturn(UserRole.EINHEITLICHER_ANSPRECHPARTNER).when(processor).getUserRole();

			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLink(linkRel)).isEmpty();
		}
	}

	@Nested
	class TestRoleVerwaltungPoststelle {
		@BeforeEach
		void init() {
			when(userService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(Boolean.FALSE);
			when(userService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(Boolean.TRUE);
		}

		@ParameterizedTest
		@EnumSource()
		void shouldNotBePresent(VorgangStatus status) {
			var processed = processor.process(buildVorgangInStatus(status));

			assertThat(processed.getLinks()).isEmpty();
		}
	}

	private EntityModel<Vorgang> buildVorgangInStatus(VorgangStatus status) {
		return EntityModel.of(VorgangHeaderTestFactory.createBuilder().status(status).build());
	}

}