/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.vorgang.forwarding;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import jakarta.validation.ConstraintValidatorContext;
import jakarta.validation.ConstraintValidatorContext.ConstraintViolationBuilder;

import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.vorgang.RedirectRequestTestFactory;

class ForwardingPasswordValidatorTest {

	@Spy
	@InjectMocks
	private ForwardingPasswordValidator validator = new ForwardingPasswordValidator();
	@Mock
	private ConstraintValidatorContext context;
	@Mock
	private ForwardingLandesnetzInfoService landesnetzInfoService;

	@DisplayName("Is valid")
	@Nested
	class TestIsValid {

		@DisplayName("with null/empty")
		@Nested
		class TestOnNullOrEmptyEmail {

			@Test
			void shouldReturnTrue() {
				var isValid = validator.isValid(RedirectRequestTestFactory.createBuilder().email(StringUtils.EMPTY).build(), context);

				assertThat(isValid).isTrue();
			}
		}

		@DisplayName("for a doman in landesnetz")
		@Nested
		class TestOnIsInLandesnetz {

			@BeforeEach
			void mock() {
				when(landesnetzInfoService.isEmailInLandesnetz(anyString())).thenReturn(true);
			}

			@Test
			void shouldNotDoPasswordValidation() {
				validator.isValid(RedirectRequestTestFactory.create(), context);

				verify(validator, never()).isValidPassword(any());
			}
		}

		@DisplayName("for domain NOT in landesnetz")
		@Nested
		class TestOnIfNOTInLandesnetz {

			@Mock
			private ConstraintViolationBuilder contraintBuilder;

			@BeforeEach
			void mock() {
				when(landesnetzInfoService.isEmailInLandesnetz(anyString())).thenReturn(false);
				doNothing().when(validator).prepareConstraint(any());
			}

			@Test
			void shouldDoPasswordValidation() {
				validator.isValid(RedirectRequestTestFactory.create(), context);

				verify(validator).isValidPassword(any());
			}
		}

		@DisplayName("password")
		@Nested
		class TestPasswordValidation {
			@Test
			void shouldReturnFalseOnLowerSize() {
				var isValid = isValid("notEnou".toCharArray());

				assertThat(isValid).isFalse();
			}

			@Test
			void shouldReturnFalseOnLargerSize() {
				var isValid = isValid(RandomStringUtils.randomAlphabetic(41).toCharArray());

				assertThat(isValid).isFalse();
			}

			@Test
			void shouldReturnFalseOnNull() {
				var isValid = isValid(null);

				assertThat(isValid).isFalse();
			}

			@Test
			void shouldReturnTrueOnValidSize() {
				var isValid = isValid(RedirectRequestTestFactory.PASSWORD);

				assertThat(isValid).isTrue();
			}

			private boolean isValid(char[] password) {
				return validator.isValidPassword(RedirectRequest.builder().password(password).build());
			}
		}
	}
}